#ifndef RUNNER_HPP
#define RUNNER_HPP


#include <string>
#include <vector>
#include <map>

namespace runner {

struct sSSHInfo {
    std::string host;
    std::string user;
    std::string port;
};

class runner {
    public:
        runner(std::string command, std::vector<std::string> args={}, std::string working_dir="", std::map<std::string, std::string> env={}, bool silent=false) :
            mCommand(command),
            mArgs(args),
            mWorkingDir(working_dir),
            mEnv(env),
            mSilent(silent),
            mReturnCode(0) {}
        virtual ~runner() {}

        virtual bool execute() = 0;

        int return_code() const { return mReturnCode; }
    protected:
        std::string mCommand;
        std::vector<std::string> mArgs;
        std::string mWorkingDir;
        std::map<std::string, std::string> mEnv;
        bool mSilent;
        int mReturnCode;
};

class runner_local : public runner {
    public:
        runner_local(std::string command, std::vector<std::string> args={}, std::string working_dir="", std::map<std::string, std::string> env={}, bool silent=false) : 
            runner(command, args, working_dir, env, silent) {}
        bool execute() override;
};

class runner_local_interactive : public runner {
    public:
        runner_local_interactive(std::string command, std::vector<std::string> args={}, std::string working_dir="", std::map<std::string, std::string> env={}) : 
            runner(command, args, working_dir, env, false) {}
        bool execute() override;
};

class runner_ssh : public runner {
    public:
        runner_ssh(const sSSHInfo sshinfo, std::string command, std::vector<std::string> args={}, std::string working_dir="", std::map<std::string, std::string> env={}, bool silent=false) : 
            runner(command, args, working_dir, env, silent), mSSHInfo(sshinfo) {}   
        bool execute() override;
    protected:
        const sSSHInfo mSSHInfo;
};

class runner_ssh_interactive : public runner_ssh {
    public:
        runner_ssh_interactive(const sSSHInfo sshinfo, std::string command, std::vector<std::string> args={}, std::string working_dir="", std::map<std::string, std::string> env={}, bool silent=true) : 
            runner_ssh(sshinfo, command, args, working_dir, env, silent) {}
        bool execute() override;
};

class runner_ssh_capture : public runner_ssh {
    public:
        runner_ssh_capture(const sSSHInfo sshinfo, std::string output, std::string command, std::vector<std::string> args={}, std::string working_dir="", std::map<std::string, std::string> env={}) : 
            runner_ssh(sshinfo, command, args, working_dir, env, true), mOutput(output) {}
        bool execute() override;
    protected:
        std::string & mOutput;
};

} // namespace runner

#endif // RUNNER_HPP